
# q-expansion of newform 804.2.a.b, downloaded from the LMFDB on 16 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 804
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [0], [0], [-2], [-4], [-3], [5], [1], [-7], [-4], [7], [-8], [-2], [7], [-14], [-15], [-6], [1], [0], [9], [0], [-4], [5], [12], [8], [20], [-11], [-12], [-2], [13], [4], [6], [14], [11], [1], [-11], [13], [-12], [-11], [-2], [-5], [-16], [25], [18], [21], [-12], [5], [-1], [-2], [-24], [24], [-7], [2], [-7], [8], [26], [-2], [-18], [-2], [-5], [6], [-23], [-18], [6], [2], [12], [-22], [-26], [-2], [24], [3], [-4], [-24], [-22], [4], [-10], [17], [-30], [-24], [-21], [-17], [-13], [28], [1], [36], [9], [-3], [-21], [-28], [-36], [27], [12], [31], [-42], [16], [39], [22], [-17], [-32], [-6], [-14], [-8], [3], [-17], [-4], [0], [18], [14], [11], [32], [35], [37], [-36], [30], [-48], [12], [6], [18], [-21], [-20], [-32], [42], [-26], [-19], [-18], [22], [41], [-26], [-38], [-12], [-24], [21], [42], [-23], [-22], [31], [20], [2], [44], [18], [21], [-51], [9], [49], [-31], [33], [24], [28], [21], [-47], [-15], [28], [-51], [-13], [-23], [-2], [-36], [-40], [-26], [-35], [-57], [40], [-3], [-39], [16], [2], [-18]]
